from requests import Request, Response, Session
from bs4 import BeautifulSoup
from .const import HTTP_READ_TIMEOUT
import logging


class HelenSession:
    HELEN_LOGIN_HOST = "https://login.helen.fi"
    TUPAS_LOGIN_URL = "https://www.helen.fi/hcc/TupasLoginFrame?service=account&locale=fi"

    _session: Session = None

    def login(self, username, password):
        """Login to Oma Helen web and follow redirects until the main page is reached.
        Will save the necessary `access-token` into a Session, which can be accessed
        with the `get_access_token()` method.

        :param username: The username for Oma Helen web service.
        :param password: The password for Oma Helen web service.
        :return: HelenSession.
        :rtype: .HelenSession
        """
        self._session = Session()
        logging.debug("Logging in to Oma Helen")
        try:
            login_response = self._send_login_request(username, password)
            self._proceed_to_main_page_from_login_response(login_response)
        except Exception as exception:
            logging.exception("Login to Oma Helen failed. Check your credentials!")
            self._session.close()
            raise exception
        logging.debug("Logged in to Oma Helen")
        return self

    def get_access_token(self):
        """Get the access-token to use the Helen API. It is required to login before the 
        token can be accessed
        """
        if self._session is None:
            raise Exception("The session is not active. Log in first")

        access_token = self._session.cookies.get("access-token")

        if access_token is None:
            raise Exception("No access token found. Log in first")

        return access_token

    def close(self):
        """Close down the session for the Oma Helen web service
        """
        if self._session is not None:
            self._session.close()
            logging.debug("HelenSession was closed")

    def _follow_redirects(self, response: Response):
        location_header = response.headers.get("Location")
        if location_header is not None:
            response = self._follow_redirects(self._session.get(location_header, timeout=HTTP_READ_TIMEOUT))
            return response
        return response

    def _make_url_request(self, url: str, method: str, data=None, params=None):
        request = Request(method, url)

        if data is not None:
            request.data = data
        if params is not None:
            request.params = params
        prepared_request = self._session.prepare_request(request)

        response = self._session.send(prepared_request, timeout=HTTP_READ_TIMEOUT)

        response = self._follow_redirects(response)

        return response

    def _get_html_input_value(self, soup: BeautifulSoup, attribute_name: str):
        return soup.find("input", {"name": attribute_name}).get("value")

    def _get_html_form_url(self, soup: BeautifulSoup):
        return soup.find("form").attrs['action']

    def _get_html_form_method(self, soup: BeautifulSoup):
        return soup.find("form").attrs["method"]

    def _get_tupas_response(self):
        return self._session.get(self.TUPAS_LOGIN_URL, timeout=HTTP_READ_TIMEOUT)

    def _send_login_request(self, username, password):
        tupas_response = self._get_tupas_response()
        tupas_soup = BeautifulSoup(tupas_response.text, "html.parser")
        authorization_url = self._get_html_form_url(tupas_soup)
        authorization_form_method = self._get_html_form_method(tupas_soup)
        authorization_response = self._make_url_request(
            authorization_url, authorization_form_method)
        authorization_soup = BeautifulSoup(
            authorization_response.text, "html.parser")
        login_url = self.HELEN_LOGIN_HOST + \
            self._get_html_form_url(authorization_soup)

        login_payload = {"username": username, "password": password}
        return self._make_url_request(login_url, "POST", login_payload)

    def _proceed_to_main_page_from_login_response(self, response: Response):
        access_granted_soup = BeautifulSoup(response.text, "html.parser")
        continue_url = self._get_html_form_url(access_granted_soup)
        continue_param_code = self._get_html_input_value(
            access_granted_soup, "code")
        continue_param_state = self._get_html_input_value(
            access_granted_soup, "state")
        continue_params = {"code": continue_param_code,
                           "state": continue_param_state}
        proceed_link_page_response = self._make_url_request(
            continue_url, "GET", params=continue_params)

        proceed_link_page_soup = BeautifulSoup(
            proceed_link_page_response.text, "html.parser")
        proceed_link_page_link_url = proceed_link_page_soup.find(
            "a").attrs['href']
        auth_response = self._make_url_request(
            proceed_link_page_link_url, "GET")

        auth_response_soup = BeautifulSoup(auth_response.text, "html.parser")
        auth_response_url = self._get_html_form_url(auth_response_soup)
        auth_response_param_code = self._get_html_input_value(
            auth_response_soup, "code")
        auth_response_param_state = self._get_html_input_value(
            auth_response_soup, "state")
        auth_response_params = {
            "code": auth_response_param_code, "state": auth_response_param_state}

        self._make_url_request(auth_response_url, "GET", params=auth_response_params)



from typing import List


class Measurement(object):
    def __init__(self, value: float, status: str):
        self.value = value
        self.status = status


class MeasurementData(object):
    def __init__(self, start: str, stop: str, resolution_s: str, resolution: str, unit: str, measurements: List):
        self.start = start
        self.stop = stop
        self.resolution_s = resolution_s
        self.resolution = resolution
        self.unit = unit
        self.measurements = list(map(lambda m: Measurement(**m), measurements))


class MeasurementIntervals(object):
    def __init__(self, electricity: List):
        self.electricity = list(map(lambda e: MeasurementData(**e), electricity))


class MeasurementResponse(object):
    def __init__(self, intervals: dict):
        self.intervals = MeasurementIntervals(**intervals) if intervals is not None else None


class SpotPricesResponse(object):
    def __init__(self, interval: dict):
        self.interval = MeasurementData(**interval) if interval is not None else None


from cachetools import cached, TTLCache
import json
from datetime import date, datetime, timedelta
from .api_response import MeasurementResponse, SpotPricesResponse
from .const import HTTP_READ_TIMEOUT
from .helen_session import HelenSession
from requests import get
from dateutil.relativedelta import relativedelta


# TODO: consider moving all calculation functions somewhere else - they are not related to HelenApiClient
class HelenApiClient:
    HELEN_API_URL = "https://api.omahelen.fi/v7"
    HELEN_API_URL_V8 = "https://api.omahelen.fi/v8"
    MEASUREMENTS_ENDPOINT = "/measurements/electricity"
    SPOT_PRICES_ENDPOINT = MEASUREMENTS_ENDPOINT + "/spot-prices"
    CONTRACT_ENDPOINT = "/contract/list"

    _latest_login_time: datetime = None
    _session: HelenSession = None  
    _margin: float = None

    def __init__(self, tax: float = None, margin: float = None):
        self._tax = 0.24 if tax is None else tax
        self._margin = 0.38 if margin is None else margin

    def login(self, username, password):
        """Login to Oma Helen. Creates a new session when called."""
        self._session = HelenSession().login(username, password)
        self._latest_login_time = datetime.now()
        return self

    def is_session_valid(self):
        """If the latest login has happened within the last hour, then the session should be valid and ready to go"""
        if self._latest_login_time is None:
            return False
        now = datetime.now()
        is_latest_login_within_hour = now-timedelta(hours=1) <= self._latest_login_time <= now
        return is_latest_login_within_hour

    def close(self):
        if self._session is not None:
            self._session.close()

    def _get_hourly_consumption_costs(self, start_date: date, end_date: date) -> list:
        hourly_prices_response = self.get_hourly_spot_prices_between_dates(start_date, end_date)
        if not hourly_prices_response.interval: return []
        # retain the hourly-price/hourly-measurement pairs (list ordering matters!) by assigning invalid items None
        hourly_prices = list(map(lambda price: price if price.status == 'valid' else None, hourly_prices_response.interval.measurements))
        hourly_measurements_data = self.get_hourly_measurements_between_dates(start_date, end_date).intervals.electricity
        if not hourly_measurements_data: return []
        hourly_measurements = list(map(lambda measurement: measurement if measurement.status == 'valid' else None, hourly_measurements_data[0].measurements))
        length = min(hourly_prices.__len__(), hourly_measurements.__len__())
        if length == 0: return []
        hourly_consumption_costs = []
        for i in range(length):
            hourly_price = hourly_prices[i]
            hourly_measurement = hourly_measurements[i]
            if hourly_price is None or hourly_measurement is None: continue
            hourly_price_with_tax_and_margin = hourly_price.value+self._margin
            hourly_consumption_costs.append((abs(hourly_price_with_tax_and_margin*hourly_measurement.value)))
        return hourly_consumption_costs

    def calculate_total_costs_by_spot_prices_between_dates(self, start_date: date, end_date: date):
        """Calculate your total electricity cost with according spot prices by hourly precision

        Returns the price in euros
        """
        hourly_consumption_costs = self._get_hourly_consumption_costs(start_date, end_date)
        total_price = sum(hourly_consumption_costs)
        total_price_with_tax_in_euros = total_price*(1+self._tax)/100
        return total_price_with_tax_in_euros

    def calculate_impact_of_usage_between_dates(self, start_date: date, end_date: date) -> float:
        """Calculate the price impact of your usage based on hourly consumption and hourly spot prices

        The price impact increases or decreases your contract's unit price in certain contracts
        such as the Helen Smart Electricity Guarantee contract
        https://www.helen.fi/en/electricity/electricity-products-and-prices/smart-electricity-guarantee
        A negative number decreases and a positive number increases the base price.

        According to Helen, the impact is calculated with formula (A-B) / E = c/kWh, where
        A = the sum of hourly consumption multiplied with the hourly price (i.e. your weighted average price of each hour)
        B = total consumption multiplied with the whole month's average market price (i.e. your average price of the whole month)
        E = total consumption
        """
        hourly_prices_response = self.get_hourly_spot_prices_between_dates(start_date, end_date)
        if not hourly_prices_response.interval: return 0.0
        # retain the hourly-price/hourly-measurement pairs (list ordering matters!) by assigning invalid items None
        hourly_prices = list(map(lambda price: price if price.status == 'valid' else None, hourly_prices_response.interval.measurements))
        hourly_measurements_response = self.get_hourly_measurements_between_dates(start_date, end_date)
        if not hourly_measurements_response.intervals or not hourly_measurements_response.intervals.electricity: return 0.0
        hourly_measurements = list(map(lambda measurement: measurement if measurement.status == 'valid' else None, hourly_measurements_response.intervals.electricity[0].measurements))
        length = min(hourly_prices.__len__(), hourly_measurements.__len__())
        if length == 0: return 0.0
        hourly_prices_without_nones = list(filter(lambda price: price is not None, hourly_prices))
        hourly_measurements_without_nones = list(filter(lambda measurement: measurement is not None, hourly_measurements))
        hourly_weighted_consumption_prices = []
        for i in range(length):
            hourly_price = hourly_prices[i]
            hourly_measurement = hourly_measurements[i]
            if hourly_price is None or hourly_measurement is None: continue
            hourly_weighted_consumption_prices.append((abs(hourly_price.value*hourly_measurement.value)))
        if not hourly_weighted_consumption_prices: return 0.0
        monthly_average_price = sum(map(lambda price: abs(price.value), hourly_prices_without_nones))/hourly_prices_without_nones.__len__()
        total_consumption = sum(map(lambda measurement: abs(measurement.value), hourly_measurements_without_nones))
        total_hourly_weighted_consumption_prices = sum(hourly_weighted_consumption_prices)
        total_consumption_average_price = monthly_average_price*total_consumption
    
        impact = (total_hourly_weighted_consumption_prices-total_consumption_average_price)/total_consumption
    
        return impact
            
    @cached(cache=TTLCache(maxsize=4, ttl=3600))
    def get_daily_measurements_between_dates(self, start: date, end: date) -> MeasurementResponse:
        """Get electricity measurements for each day of the wanted month of the on-going year."""

        previous_day = start + relativedelta(days=-1)
        start_time = f"{previous_day}T22:00:00+00:00"
        end_time = f"{end}T21:59:59+00:00"
        delivery_site_id = self.get_delivery_site_id()
        measurements_params = {
            "begin": start_time,
            "end": end_time,
            "resolution": "day",
            "delivery_site_id": delivery_site_id,
            "allow_transfer": "true"
        }

        measurements_url = self.HELEN_API_URL + self.MEASUREMENTS_ENDPOINT
        response_json_text = get(
            measurements_url, measurements_params, headers=self._api_request_headers(), timeout=HTTP_READ_TIMEOUT).text
        daily_measurement: MeasurementResponse = MeasurementResponse(
            **json.loads(response_json_text))

        return daily_measurement

    @cached(cache=TTLCache(maxsize=2, ttl=3600))
    def get_monthly_measurements_by_year(self, year: int) -> MeasurementResponse:
        """Get electricity measurements for each month of the on-going year."""
        
        last_year = year-1
        start_time = f"{last_year}-12-31T22:00:00+00:00"
        end_time = f"{year}-12-31T21:59:59+00:00"
        delivery_site_id = self.get_delivery_site_id()
        measurements_params = {
            "begin": start_time,
            "end": end_time,
            "resolution": "month",
            "delivery_site_id": delivery_site_id,
            "allow_transfer": "true"
        }

        measurements_url = self.HELEN_API_URL + self.MEASUREMENTS_ENDPOINT
        response_json_text = get(
            measurements_url, measurements_params, headers=self._api_request_headers(), timeout=HTTP_READ_TIMEOUT).text
        monthly_measurement: MeasurementResponse = MeasurementResponse(
            **json.loads(response_json_text))

        return monthly_measurement


    @cached(cache=TTLCache(maxsize=4, ttl=3600))
    def get_hourly_measurements_between_dates(self, start: date, end: date) -> MeasurementResponse:
        """Get electricity measurements for each hour between given dates."""
        
        previous_day = start + relativedelta(days=-1)
        start_time = f"{previous_day}T22:00:00+00:00"
        end_time = f"{end}T21:59:59+00:00"
        delivery_site_id = self.get_delivery_site_id()
        measurements_params = {
            "begin": start_time,
            "end": end_time,
            "resolution": "hour",
            "delivery_site_id": delivery_site_id,
            "allow_transfer": "true"
        }

        measurements_url = self.HELEN_API_URL + self.MEASUREMENTS_ENDPOINT
        response_json_text = get(
            measurements_url, measurements_params, headers=self._api_request_headers(), timeout=HTTP_READ_TIMEOUT).text
        hourly_measurement: MeasurementResponse = MeasurementResponse(
            **json.loads(response_json_text))

        return hourly_measurement


    @cached(cache=TTLCache(maxsize=4, ttl=3600))
    def get_hourly_spot_prices_between_dates(self, start: date, end: date) -> SpotPricesResponse:
        """Get electricity spot prices for each hour between given dates."""
        
        previous_day = start + relativedelta(days=-1)
        start_time = f"{previous_day}T22:00:00+00:00"
        end_time = f"{end}T21:59:59+00:00"
        delivery_site_id = self.get_delivery_site_id()
        spot_prices_params = {
            "begin": start_time,
            "end": end_time,
            "resolution": "hour",
            "delivery_site_id": delivery_site_id,
            "allow_transfer": "true"
        }

        spot_prices_url = self.HELEN_API_URL_V8 + self.SPOT_PRICES_ENDPOINT
        response_json_text = get(
            spot_prices_url, spot_prices_params, headers=self._api_request_headers(), timeout=HTTP_READ_TIMEOUT).text
        spot_prices_measurement: SpotPricesResponse = SpotPricesResponse(
            **json.loads(response_json_text))

        return spot_prices_measurement
    
    @cached(cache=TTLCache(maxsize=2, ttl=3600))
    def get_contract_data_json(self):
        """Get your contract data."""

        contract_url = self.HELEN_API_URL + self.CONTRACT_ENDPOINT
        contract_response_dict = get(contract_url, headers=self._api_request_headers(), timeout=HTTP_READ_TIMEOUT).json()
        self._contract_data_dict = contract_response_dict
        return contract_response_dict

    def get_delivery_site_id(self) -> int:
        """Get the delivery site id from your contract data."""

        return self.get_contract_data_json()[0]["delivery_site"]["id"]

    def get_contract_base_price(self) -> int:
        """Get the contract base price from your contract data."""
        
        contract_data = self.get_contract_data_json()
        contract_components = contract_data[0]["products"][0]["components"]
        base_price_component = next(filter(lambda component: component["is_base_price"], contract_components))
        return base_price_component["price"]

    def get_api_access_token(self):
        return self._session.get_access_token()

    def _api_request_headers(self):
        return {
            "Authorization": f"Bearer {self.get_api_access_token()}",
            "Accept": "application/json"
        }

    def set_margin(self, margin: float):
        self._margin = margin



from cmd import Cmd
from datetime import date, datetime
from .api_client import HelenApiClient
from getpass import getpass
from .price_client import HelenPriceClient
from .utils import get_month_date_range_by_date
import json

def _json_serializer(value):
    if isinstance(value, datetime):
        return value.strftime("%Y%m%d%H%M%S")
    else:
        return value.__dict__

class HelenCLIPrompt(Cmd):
    prompt = "helen-cli> "
    intro = "Type ? to list commands"

    helen_price_client = HelenPriceClient()

    tax = 0.1 # 10%
    margin = helen_price_client.get_exchange_prices().margin
    api_client = HelenApiClient(tax, margin)


    def __init__(self, username, password):
        super(HelenCLIPrompt, self).__init__()
        self.api_client.login(username, password)

    def do_exit(self, input=None):
        """Exit the CLI"""

        self.api_client.close()
        print("Bye")
        return True

    def do_calculate_spot_cost_between_dates(self, input=None):
        """Calculate the price of your Exchange Electricity (spot) contract between a start date and an end date
        The provided dates should be presented in format 'YYYY-mm-dd'

        Includes 10% tax. The price returned is in euros.

        Usage example:
        calculate_spot_cost_between_dates 2022-12-01 2022-12-31
        """

        if input is None:
            print("Please provide proper start and end dates in format 'YYYY-mm-dd'")
        else:
            try:
                start_date_str, end_date_str = str(input).split(' ')
                start_date = datetime.strptime(start_date_str, '%Y-%m-%d').date()
                end_date = datetime.strptime(end_date_str, '%Y-%m-%d').date()
                if start_date > end_date: 
                    print("Start date must be before end date")
                    raise ValueError()
                price = self.api_client.calculate_total_costs_by_spot_prices_between_dates(start_date, end_date)
                print(price)
            except ValueError:
                print("Please provide proper start and end dates in format 'YYYY-mm-dd'")

    def do_calculate_the_impact_of_usage_between_dates(self, input=None):
        """Calculate the impact of usage for Helen Smart Electricity Guarantee contract between a start date and an end date
        The provided dates should be presented in format 'YYYY-mm-dd'

        Usage example:
        calculate_the_impact_of_usage_between_dates 2022-12-01 2022-12-31
        """

        if input is None:
            print("Please provide proper start and end dates in format 'YYYY-mm-dd'")
        else:
            try:
                start_date_str, end_date_str = str(input).split(' ')
                start_date = datetime.strptime(start_date_str, '%Y-%m-%d').date()
                end_date = datetime.strptime(end_date_str, '%Y-%m-%d').date()
                if start_date > end_date: 
                    print("Start date must be before end date")
                    raise ValueError()
                impact = self.api_client.calculate_impact_of_usage_between_dates(start_date, end_date)
                print(impact)
            except ValueError:
                print("Please provide proper start and end dates in format 'YYYY-mm-dd'")

    def do_get_monthly_measurements_json(self, input=None):
        """Get the monthly electricity measurements of the on-going year as JSON"""

        year = date.today().year
        monthly_measurements = self.api_client.get_monthly_measurements_by_year(year)
        monthly_measurements_json = json.dumps(monthly_measurements, default=lambda o: o.__dict__, indent=2)
        print(monthly_measurements_json)

    def do_get_daily_measurements_json(self, input=None):
        """Get the daily electricity measurements of the on-going month of the on-going year as JSON"""

        previous_month_last_day_date, wanted_month_last_day_date = get_month_date_range_by_date(date.today())

        daily_measurements = self.api_client.get_daily_measurements_between_dates(previous_month_last_day_date, wanted_month_last_day_date)
        daily_measurements_json = json.dumps(daily_measurements, default=lambda o: o.__dict__, indent=2)
        print(daily_measurements_json)

    def do_get_contract_data_json(self, input=None):
        """Get the whole contract data as JSON"""

        contract_data_json = self.api_client.get_contract_data_json()
        contract_data_json_pretty = json.dumps(contract_data_json, default=lambda o: o.__dict__, indent=2)
        print(contract_data_json_pretty)

    def do_get_market_prices_json(self, input=None):
        """Get prices for the Market Price contract type as JSON"""

        price = self.helen_price_client.get_market_price_prices()
        price_json = json.dumps(price, default=_json_serializer, indent=2)
        print(price_json)

    def do_get_smart_guarantee_price_json(self, input=None):
        """Get price for the Smart Guarantee contract type as JSON"""

        price = self.helen_price_client.get_smart_guarantee_prices()
        price_json = json.dumps(price, default=_json_serializer, indent=2)
        print(price_json)

    def do_get_exchange_margin_price_json(self, input=None):
        """Get margin price for the Exchange Electricity contract type as JSON"""

        price = self.helen_price_client.get_exchange_prices()
        price_json = json.dumps(price, default=_json_serializer, indent=2)
        print(price_json)

    def do_get_contract_delivery_site_id(self, input=None):
        """Helper to get the delivery site id from your contract data. To see the whole contract data as JSON, use get_contract_data_json"""
        
        site_id = self.api_client.get_delivery_site_id()
        print(site_id)

    def do_get_contract_base_price(self, input=None):
        """Helper to get the contract base price from your contract data. To see the whole contract data as JSON, use get_contract_data_json"""
        
        base_price = self.api_client.get_contract_base_price()
        print(base_price)

    def do_get_api_access_token(self, input=None):
        """Get your access token for the Oma Helen API."""
        
        access_token = self.api_client.get_api_access_token()
        print(access_token)


def main():
    print("Log in to Oma Helen")
    username = input("Username: ")
    password = getpass()
    HelenCLIPrompt(username, password).cmdloop()


if __name__ == "__main__":
    main()



from datetime import date, datetime, timedelta
from .const import HTTP_READ_TIMEOUT
from requests import get
from bs4 import BeautifulSoup


class VattenfallPriceClient:
    HOURLY_PRICES_URL = "https://www.vattenfall.fi/api/price/spot/{date}/{date}?lang=fi"
    DAILY_AVERAGE_PRICES_URL = "https://www.vattenfall.fi/api/price/spot/average/{start_date}/{end_date}?lang=fi"
    HEADERS = { 'User-Agent': 'Mozilla/5.0' }
    
    def get_hourly_prices_for_day(self, day: date):
        url = self.HOURLY_PRICES_URL.format(date=day)
        response = get(url, headers=self.HEADERS)
        if response.status_code == 200:
            return response.json()
        else:
            print("Could not fetch prices. Response code was: " + str(response.status_code))

    def get_daily_average_prices_between_dates(self, start_date: date, end_date: date):
        url = self.DAILY_AVERAGE_PRICES_URL.format(start_date=start_date, end_date=end_date)
        response = get(url, headers=self.HEADERS)
        if response.status_code == 200:
            return response.json()
        else:
            print("Could not fetch prices. Response code was: " + str(response.status_code))


class HelenPrices:
    def __init__(self):
        self.timestamp = datetime.now()


class HelenMarketPrices(HelenPrices):
    def __init__(self, last_month, current_month, next_month):
        super().__init__()
        self.last_month: float = last_month
        self.current_month: float = current_month
        self.next_month: float = next_month


class HelenSmartGuaranteePrices(HelenPrices):
    def __init__(self, price):
        super().__init__()
        self.price: float = price


class HelenExchangePrices(HelenPrices):
    def __init__(self, margin):
        super().__init__()
        self.margin: float = margin


class HelenPriceClient:
    MARKET_PRICE_ELECTRICITY_URL = "https://www.helen.fi/sahko/sahkosopimus/markkinahinta"
    SMART_ELECTRICITY_GUARANTEE_URL = "https://www.helen.fi/sahko/sahkosopimus/fiksusahko-takuu"
    EXCHANGE_ELECTRICITY_URL = "https://www.helen.fi/sahko/sahkosopimus/porssisahko"

    _helen_market_price_prices: HelenMarketPrices = None
    _helen_smart_guarantee_prices: HelenSmartGuaranteePrices = None
    _helen_exchange_prices: HelenExchangePrices = None
        

    def _are_market_price_prices_valid(self):
        return self._is_helen_prices_valid(self._helen_market_price_prices)


    def _are_smart_guarantee_prices_valid(self):
        return self._is_helen_prices_valid(self._helen_smart_guarantee_prices)


    def _are_exchange_prices_valid(self):
        return self._is_helen_prices_valid(self._helen_exchange_prices)

    
    def _is_helen_prices_valid(self, helen_prices: HelenPrices):
        """If the latest price scrape has happened within the last hour, then use cache"""

        now = datetime.now()
        if helen_prices is None:
            return False
        were_market_prices_scraped_within_hour = now-timedelta(hours=1) <= helen_prices.timestamp <= now
        return were_market_prices_scraped_within_hour


    def _scrape_market_price_prices(self):
        kwh_substring = " c/kWh"

        price_site_response = get(self.MARKET_PRICE_ELECTRICITY_URL, timeout=HTTP_READ_TIMEOUT)
        price_site_soup = BeautifulSoup(price_site_response.text, "html.parser")

        element = price_site_soup.select_one(f'td:-soup-contains("{kwh_substring}")')
        last_month_price = element.text # td -> <text>
        if kwh_substring in last_month_price:
            last_month_price = last_month_price[:last_month_price.index(kwh_substring)]
            last_month_price = float(last_month_price.replace(",", "."))

        element = element.find_next_sibling()
        current_month_price = element.next_element.text # td -> strong -> <text>
        if kwh_substring in current_month_price:
            current_month_price = current_month_price[:current_month_price.index(kwh_substring)]
            current_month_price = float(current_month_price.replace(",", "."))

        element = element.find_next_sibling()
        next_month_price = element.text # td -> <text>
        if kwh_substring in next_month_price:
            next_month_price = next_month_price[:next_month_price.index(kwh_substring):]
            next_month_price = float(next_month_price.replace(",", "."))
        else: 
            next_month_price = None

        return last_month_price, current_month_price, next_month_price


    def get_market_price_prices(self) -> HelenMarketPrices:
        """Get the prices for last month, current month and next month for the Market Price contract"""
        if self._are_market_price_prices_valid():
            return self._helen_market_price_prices
        last_month_price, current_month_price, next_month_price = self._scrape_market_price_prices()

        self._helen_market_price_prices = HelenMarketPrices(last_month_price, current_month_price, next_month_price)
        return self._helen_market_price_prices


    def _scrape_smart_guarantee_prices(self):
        price_site_response = get(self.SMART_ELECTRICITY_GUARANTEE_URL, timeout=HTTP_READ_TIMEOUT)
        price_site_soup = BeautifulSoup(price_site_response.text, "html.parser")

        element = price_site_soup.select_one(f'span.product-info-block__data--price')
        price = element.text

        return float(price.replace(",", "."))


    def get_smart_guarantee_prices(self) -> HelenSmartGuaranteePrices:
        """Get the current price (c/kwh) for the Smart Guarantee contract"""
        if self._are_smart_guarantee_prices_valid():
            return self._helen_smart_guarantee_prices
        price = self._scrape_smart_guarantee_prices()

        self._helen_smart_guarantee_prices = HelenSmartGuaranteePrices(price)
        return self._helen_smart_guarantee_prices


    def _scrape_exchange_prices(self):
        price_site_response = get(self.EXCHANGE_ELECTRICITY_URL, timeout=HTTP_READ_TIMEOUT)
        price_site_soup = BeautifulSoup(price_site_response.text, "html.parser")

        element = price_site_soup.select_one(f'span.product-info-block__data--price')
        margin = element.text

        return float(margin.replace(",", "."))


    def get_exchange_prices(self) -> HelenExchangePrices:
        """Get the margin price for the Exchange Electricity contract"""
        if self._are_exchange_prices_valid():
            return self._helen_exchange_prices
        margin = self._scrape_exchange_prices()

        self._helen_exchange_prices = HelenExchangePrices(margin)
        return self._helen_exchange_prices


import calendar
from datetime import date

def get_month_date_range_by_date(date_param: date) -> tuple[date, date]:
    """
    Get the start date and end date of the month of the given date.
    """
    wanted_month_first_day = date_param.replace(day=1)
    wanted_month_last_day = date_param.replace(day=calendar.monthrange(wanted_month_first_day.year, wanted_month_first_day.month)[-1])

    return wanted_month_first_day, wanted_month_last_day


#!/usr/bin/env python

"""The setup script."""

import pathlib
from setuptools import setup, find_packages

here = pathlib.Path(__file__).parent.resolve()
long_description = (here / "README.md").read_text(encoding="utf-8")

requirements = [
    'beautifulsoup4==4.11.1',
    'python-dateutil==2.8.2',
    'requests>=2.28.1',
    'soupsieve==2.3.2.post1',
    'urllib3==1.26.12',
    'cachetools==5.2.0'
    ]

setup(
    author="carohauta",
    author_email='carosoft.dev@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    description="Oma Helen API library and CLI",
    entry_points={
        'console_scripts': [
            'oma-helen-cli=helenservice.cli:main',
        ],
    },
    name="oma-helen-cli",
    install_requires=requirements,
    license="MIT license",
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    packages=find_packages(include=['helenservice', 'helenservice.*']),
    url='https://github.com/carohauta/oma-helen-cli',
    version='1.0.2',
    zip_safe=False,
)
